@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2009-2011,2013-2016                      Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,30 +25,17 @@
 #  define NITER	2048
 #endif
 
-int token = 42;
-starpu_data_handle_t token_handle;
-
 #ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
+extern void increment_cuda(void *descr[], void *_args);
 #endif
 
-void increment_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void increment_cpu(void *descr[], void *_args)
 {
+	(void)_args;
 	int *tokenptr = (int *)STARPU_VECTOR_GET_PTR(descr[0]);
 	(*tokenptr)++;
 }
 
-/* Dummy cost function for simgrid */
-static double cost_function(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0.000001;
-}
-static struct starpu_perfmodel dumb_model =
-{
-	.type		= STARPU_COMMON,
-	.cost_function	= cost_function
-};
-
 static struct starpu_codelet increment_cl =
 {
 #ifdef STARPU_USE_CUDA
@@ -59,10 +44,10 @@ static struct starpu_codelet increment_cl =
 	.cpu_funcs = {increment_cpu},
 	.nbuffers = 1,
 	.modes = {STARPU_RW},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
-void increment_token(void)
+void increment_token(starpu_data_handle_t token_handle)
 {
 	struct starpu_task *task = starpu_task_create();
 
@@ -77,24 +62,30 @@ void increment_token(void)
 int main(int argc, char **argv)
 {
 	int ret, rank, size;
+	int mpi_init;
+	int token = 42;
+	starpu_data_handle_t token_handle;
+
+	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
+
+	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
-	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED);
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(NULL, NULL, 0);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
-
-	if (size < 2)
+	if (size < 2 || (starpu_cpu_worker_get_count() + starpu_cuda_worker_get_count() == 0))
 	{
 		if (rank == 0)
-			FPRINTF(stderr, "We need at least 2 processes.\n");
-
+		{
+			if (size < 2)
+				FPRINTF(stderr, "We need at least 2 processes.\n");
+			else
+				FPRINTF(stderr, "We need at least 1 CPU or CUDA worker.\n");
+		}
 		starpu_mpi_shutdown();
-		starpu_shutdown();
-		MPI_Finalize();
+		if (!mpi_init)
+			MPI_Finalize();
 		return STARPU_TEST_SKIPPED;
 	}
 
@@ -112,8 +103,10 @@ int main(int argc, char **argv)
 
 		if (loop == 0 && rank == 0)
 		{
+			starpu_data_acquire(token_handle, STARPU_W);
 			token = 0;
 			FPRINTF(stdout, "Start with token value %d\n", token);
+			starpu_data_release(token_handle);
 		}
 		else
 		{
@@ -123,7 +116,7 @@ int main(int argc, char **argv)
 			starpu_mpi_wait(&req, &status);
 		}
 
-		increment_token();
+		increment_token(token_handle);
 
 		if (loop == last_loop && rank == last_rank)
 		{
@@ -142,13 +135,14 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(token_handle);
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 
-	MPI_Finalize();
+	if (!mpi_init)
+		MPI_Finalize();
 
 #ifndef STARPU_SIMGRID
 	if (rank == last_rank)
 	{
+		FPRINTF(stderr, "[%d] token = %d == %d * %d ?\n", rank, token, nloops, size);
 		STARPU_ASSERT(token == nloops*size);
 	}
 #endif
