@@ -1,9 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016                           Inria
- * Copyright (C) 2009-2017                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2016       Uppsala University
+ * Copyright (C) 2017       Erwan Leria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,6 +37,7 @@
 #include <core/sched_ctx.h>
 #include <time.h>
 #include <signal.h>
+#include <core/simgrid.h>
 #ifdef STARPU_HAVE_WINDOWS
 #include <windows.h>
 #endif
@@ -55,6 +57,12 @@ static int limit_max_submitted_tasks;
 static int watchdog_crash;
 static int watchdog_delay;
 
+/*
+ * Function to call when watchdog detects that no task has finished for more than STARPU_WATCHDOG_TIMEOUT seconds
+ */
+static void (*watchdog_hook)(void *) = NULL;
+static void * watchdog_hook_arg = NULL;
+
 #define _STARPU_TASK_MAGIC 42
 
 /* Called once at starpu_init */
@@ -83,6 +91,7 @@ void starpu_task_init(struct starpu_task *task)
 	memset(task, 0, sizeof(struct starpu_task));
 
 	task->sequential_consistency = 1;
+	task->where = -1;
 
 	/* Now we can initialise fields which recquire custom value */
 	/* Note: remember to update STARPU_TASK_INITIALIZER as well */
@@ -98,18 +107,12 @@ void starpu_task_init(struct starpu_task *task)
 
 	task->predicted = NAN;
 	task->predicted_transfer = NAN;
+	task->predicted_start = NAN;
 
 	task->magic = _STARPU_TASK_MAGIC;
 	task->sched_ctx = STARPU_NMAX_SCHED_CTXS;
 
 	task->flops = 0.0;
-
-	task->scheduled = 0;
-	task->prefetched = 0;
-
-	task->dyn_handles = NULL;
-	task->dyn_interfaces = NULL;
-	task->name = NULL;
 }
 
 /* Free all the ressources allocated for a task, without deallocating the task
@@ -187,7 +190,7 @@ void _starpu_task_destroy(struct starpu_task *task)
 	{
 		starpu_task_clean(task);
 		/* TODO handle the case of task with detach = 1 and destroy = 1 */
-		/* TODO handle the case of non terminated tasks -> return -EINVAL */
+		/* TODO handle the case of non terminated tasks -> assertion failure, it's too dangerous to be doing something like this */
 
 		/* Does user want StarPU release cl_arg ? */
 		if (task->cl_arg_free)
@@ -239,7 +242,7 @@ int starpu_task_wait(struct starpu_task *task)
 
 	STARPU_ASSERT_MSG(_starpu_worker_may_perform_blocking_calls(), "starpu_task_wait must not be called from a task or callback");
 
-	struct _starpu_job *j = (struct _starpu_job *)task->starpu_private;
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
 
 	_STARPU_TRACE_TASK_WAIT_START(j);
 
@@ -256,6 +259,19 @@ int starpu_task_wait(struct starpu_task *task)
 	return 0;
 }
 
+int starpu_task_wait_array(struct starpu_task **tasks, unsigned nb_tasks)
+{
+	unsigned i;
+
+	for (i = 0; i < nb_tasks; i++)
+	{
+		int ret = starpu_task_wait(tasks[i]);
+		if (ret)
+			return ret;
+	}
+	return 0;
+}
+
 #ifdef STARPU_OPENMP
 int _starpu_task_test_termination(struct starpu_task *task)
 {
@@ -380,6 +396,11 @@ void _starpu_codelet_check_deprecated_fields(struct starpu_codelet *cl)
 {
 	if (!cl)
 		return;
+	if (cl->checked)
+	{
+		STARPU_RMB();
+		return;
+	}
 
 	uint32_t where = cl->where;
 	int is_where_unset = where == 0;
@@ -483,14 +504,14 @@ void _starpu_codelet_check_deprecated_fields(struct starpu_codelet *cl)
 
 	some_impl = 0;
 	for (i = 0; i < STARPU_MAXIMPLEMENTATIONS; i++)
-		if (cl->scc_funcs[i])
+		if (cl->mpi_ms_funcs[i])
 		{
 			some_impl = 1;
 			break;
 		}
 	if (some_impl && is_where_unset)
 	{
-		where |= STARPU_SCC;
+		where |= STARPU_MPI_MS;
 	}
 
 	some_impl = 0;
@@ -502,9 +523,12 @@ void _starpu_codelet_check_deprecated_fields(struct starpu_codelet *cl)
 		}
 	if (some_impl && is_where_unset)
 	{
-		where |= STARPU_MIC|STARPU_SCC;
+		where |= STARPU_MIC|STARPU_MPI_MS;
 	}
 	cl->where = where;
+
+	STARPU_WMB();
+	cl->checked = 1;
 }
 
 void _starpu_task_check_deprecated_fields(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED)
@@ -517,6 +541,11 @@ static int _starpu_task_submit_head(struct starpu_task *task)
 	unsigned is_sync = task->synchronous;
 	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
 
+	if (task->status == STARPU_TASK_STOPPED || task->status == STARPU_TASK_FINISHED)
+		task->status = STARPU_TASK_INVALID;
+	else
+		STARPU_ASSERT(task->status == STARPU_TASK_INVALID);
+
 	if (j->internal)
 	{
 		// Internal tasks are submitted to initial context
@@ -538,6 +567,8 @@ static int _starpu_task_submit_head(struct starpu_task *task)
 
 	_starpu_task_check_deprecated_fields(task);
 	_starpu_codelet_check_deprecated_fields(task->cl);
+	if (task->where== -1 && task->cl)
+		task->where = task->cl->where;
 
 	if (task->cl)
 	{
@@ -559,17 +590,28 @@ static int _starpu_task_submit_head(struct starpu_task *task)
 		for (i = 0; i < nbuffers; i++)
 		{
 			starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, i);
+			enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, i);
 			int node = task->cl->specific_nodes ? STARPU_CODELET_GET_NODE(task->cl, i) : -1;
 			/* Make sure handles are valid */
 			STARPU_ASSERT_MSG(handle->magic == _STARPU_TASK_MAGIC, "data %p is invalid (was it already unregistered?)", handle);
 			/* Make sure handles are not partitioned */
 			STARPU_ASSERT_MSG(handle->nchildren == 0, "only unpartitioned data (or the pieces of a partitioned data) can be used in a task");
 			/* Make sure the specified node exists */
-			STARPU_ASSERT_MSG(node == -1 || (node >= 0 && node < starpu_memory_nodes_get_count()), "The codelet-specified memory node does not exist");
+			STARPU_ASSERT_MSG(node == STARPU_SPECIFIC_NODE_LOCAL || node == STARPU_SPECIFIC_NODE_CPU || node == STARPU_SPECIFIC_NODE_SLOW || (node >= 0 && node < (int) starpu_memory_nodes_get_count()), "The codelet-specified memory node does not exist");
 			/* Provide the home interface for now if any,
 			 * for can_execute hooks */
 			if (handle->home_node != -1)
 				_STARPU_TASK_SET_INTERFACE(task, starpu_data_get_interface_on_node(handle, handle->home_node), i);
+			if (!(task->cl->flags & STARPU_CODELET_NOPLANS) &&
+			    ((handle->nplans && !handle->nchildren) || handle->siblings)
+			    && handle->partition_automatic_disabled == 0
+			    )
+				/* This handle is involved with asynchronous
+				 * partitioning as a parent or a child, make
+				 * sure the right plan is active, submit
+				 * appropiate partitioning / unpartitioning if
+				 * not */
+				_starpu_data_partition_access_submit(handle, (mode & STARPU_W) != 0);
 		}
 
 		/* Check the type of worker(s) required by the task exist */
@@ -642,14 +684,20 @@ int starpu_task_submit(struct starpu_task *task)
 		return ret;
 	}
 
-	if (!j->internal && !continuation)
+	if (!continuation)
+	{
+		STARPU_ASSERT_MSG(!j->submitted || j->terminated >= 1, "Tasks can not be submitted a second time before being terminated. Please use different task structures, or use the regenerate flag to let the task resubmit itself automatically.");
 		_STARPU_TRACE_TASK_SUBMIT(j,
 			_starpu_get_sched_ctx_struct(task->sched_ctx)->iterations[0],
 			_starpu_get_sched_ctx_struct(task->sched_ctx)->iterations[1]);
+	}
 
 	/* If this is a continuation, we don't modify the implicit data dependencies detected earlier. */
 	if (task->cl && !continuation)
+	{
+		_starpu_job_set_ordered_buffers(j);
 		_starpu_detect_implicit_data_deps(task);
+	}
 
 	if (bundle)
 	{
@@ -691,7 +739,8 @@ int starpu_task_submit(struct starpu_task *task)
 
 	ret = _starpu_submit_job(j);
 #ifdef STARPU_SIMGRID
-	MSG_process_sleep(0.000001);
+	if (_starpu_simgrid_task_submit_cost())
+		starpu_sleep(0.000001);
 #endif
 
 	if (is_sync)
@@ -739,6 +788,7 @@ int _starpu_task_submit_nodeps(struct starpu_task *task)
 	if (task->cl)
 		/* This would be done by data dependencies checking */
 		_starpu_job_set_ordered_buffers(j);
+	STARPU_ASSERT(task->status == STARPU_TASK_BLOCKED);
 	task->status = STARPU_TASK_READY;
 	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
@@ -779,6 +829,7 @@ int _starpu_task_submit_conversion_task(struct starpu_task *task,
 	_starpu_increment_nready_tasks_of_sched_ctx(j->task->sched_ctx, j->task->flops, j->task);
 	_starpu_job_set_ordered_buffers(j);
 
+	STARPU_ASSERT(task->status == STARPU_TASK_INVALID);
 	task->status = STARPU_TASK_READY;
 	_starpu_profiling_set_task_push_start_time(task);
 
@@ -789,6 +840,7 @@ int _starpu_task_submit_conversion_task(struct starpu_task *task,
 	struct _starpu_worker *worker;
 	worker = _starpu_get_worker_struct(workerid);
 	starpu_task_list_push_back(&worker->local_tasks, task);
+	starpu_wake_worker_locked(worker->workerid);
 
 	_starpu_profiling_set_task_push_end_time(task);
 
@@ -855,14 +907,14 @@ int _starpu_task_wait_for_all_and_return_nb_waited_tasks(void)
 			int s;
 			for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 			{
-				if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+				if(config->sched_ctxs[s].do_schedule == 1)
 				{
 					_starpu_sched_do_schedule(config->sched_ctxs[s].id);
 				}
 			}
 			for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 			{
-				if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+				if(config->sched_ctxs[s].do_schedule == 1)
 				{
 					starpu_task_wait_for_all_in_ctx(config->sched_ctxs[s].id);
 				}
@@ -925,7 +977,7 @@ int starpu_task_wait_for_n_submitted(unsigned n)
 			int s;
 			for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 			{
-				if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+				if(config->sched_ctxs[s].do_schedule == 1)
 				{
 					_starpu_wait_for_n_submitted_tasks_of_sched_ctx(config->sched_ctxs[s].id, n);
 				}
@@ -967,14 +1019,14 @@ int starpu_task_wait_for_no_ready(void)
 		int s;
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				_starpu_sched_do_schedule(config->sched_ctxs[s].id);
 			}
 		}
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				_starpu_wait_for_no_ready_of_sched_ctx(config->sched_ctxs[s].id);
 			}
@@ -1011,7 +1063,7 @@ void starpu_do_schedule(void)
 		int s;
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				_starpu_sched_do_schedule(config->sched_ctxs[s].id);
 			}
@@ -1036,7 +1088,7 @@ starpu_drivers_request_termination(void)
 		int s;
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				_starpu_check_nsubmitted_tasks_of_sched_ctx(config->sched_ctxs[s].id);
 			}
@@ -1057,7 +1109,7 @@ int starpu_task_nsubmitted(void)
 		int s;
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				nsubmitted += _starpu_get_nsubmitted_tasks_of_sched_ctx(config->sched_ctxs[s].id);
 			}
@@ -1078,7 +1130,7 @@ int starpu_task_nready(void)
 		int s;
 		for(s = 0; s < STARPU_NMAX_SCHED_CTXS; s++)
 		{
-			if(config->sched_ctxs[s].id != STARPU_NMAX_SCHED_CTXS)
+			if(config->sched_ctxs[s].do_schedule == 1)
 			{
 				nready += starpu_sched_ctx_get_nready_tasks(config->sched_ctxs[s].id);
 			}
@@ -1101,6 +1153,18 @@ void _starpu_set_current_task(struct starpu_task *task)
 	STARPU_PTHREAD_SETSPECIFIC(current_task_key, task);
 }
 
+int starpu_task_get_current_data_node(unsigned i)
+{
+	struct starpu_task *task = starpu_task_get_current();
+	if (!task)
+		return -1;
+
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
+	struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
+	unsigned orderedindex = descrs[i].orderedindex;
+	return descrs[orderedindex].node;
+}
+
 #ifdef STARPU_OPENMP
 /* Prepare the fields of the currentl task for accepting a new set of
  * dependencies in anticipation of becoming a continuation.
@@ -1173,7 +1237,7 @@ _starpu_handle_needs_conversion_task_for_arch(starpu_data_handle_t handle,
 				case STARPU_CUDA_RAM:      /* Fall through */
 				case STARPU_OPENCL_RAM:
 				case STARPU_MIC_RAM:
-				case STARPU_SCC_RAM:
+                                case STARPU_MPI_MS_RAM:
 					return 1;
 				default:
 					STARPU_ABORT();
@@ -1182,7 +1246,7 @@ _starpu_handle_needs_conversion_task_for_arch(starpu_data_handle_t handle,
 		case STARPU_CUDA_RAM:    /* Fall through */
 		case STARPU_OPENCL_RAM:
 		case STARPU_MIC_RAM:
-		case STARPU_SCC_RAM:
+		case STARPU_MPI_MS_RAM:
 			switch(starpu_node_get_kind(handle->mf_node))
 			{
 				case STARPU_CPU_RAM:
@@ -1190,7 +1254,7 @@ _starpu_handle_needs_conversion_task_for_arch(starpu_data_handle_t handle,
 				case STARPU_CUDA_RAM:
 				case STARPU_OPENCL_RAM:
 				case STARPU_MIC_RAM:
-				case STARPU_SCC_RAM:
+                                case STARPU_MPI_MS_RAM:
 					return 0;
 				default:
 					STARPU_ABORT();
@@ -1270,14 +1334,18 @@ static void *watchdog_func(void *arg)
 		if (!config->watchdog_ok && last_nsubmitted
 				&& last_nsubmitted == starpu_task_nsubmitted())
 		{
-			_STARPU_MSG("The StarPU watchdog detected that no task finished for %fs (can be configured through STARPU_WATCHDOG_TIMEOUT)\n",
-				    timeout);
+			if (watchdog_hook == NULL)
+				_STARPU_MSG("The StarPU watchdog detected that no task finished for %fs (can be configured through STARPU_WATCHDOG_TIMEOUT)\n",
+									timeout);
+			else
+				watchdog_hook(watchdog_hook_arg);
+
 			if (watchdog_crash)
 			{
 				_STARPU_MSG("Crashing the process\n");
 				raise(SIGABRT);
 			}
-			else
+			else if (watchdog_hook == NULL)
 				_STARPU_MSG("Set the STARPU_WATCHDOG_CRASH environment variable if you want to abort the process in such a case\n");
 		}
 		/* Only shout again after another period */
@@ -1287,7 +1355,13 @@ static void *watchdog_func(void *arg)
 	return NULL;
 }
 
-void _starpu_watchdog_init(void)
+void starpu_task_watchdog_set_hook(void (*hook)(void *), void *hook_arg)
+{
+	watchdog_hook = hook;
+	watchdog_hook_arg = hook_arg;
+}
+
+void _starpu_watchdog_init()
 {
 	struct _starpu_machine_config *config = _starpu_get_machine_config();
 	char *timeout_env = starpu_getenv("STARPU_WATCHDOG_TIMEOUT");
@@ -1307,5 +1381,5 @@ void _starpu_watchdog_shutdown(void)
 	if (!timeout_env)
 		return;
 
-	starpu_pthread_join(watchdog_thread, NULL);
+	STARPU_PTHREAD_JOIN(watchdog_thread, NULL);
 }
