@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2016                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,6 +31,8 @@ static starpu_data_handle_t handle;
 #ifdef STARPU_USE_CUDA
 static void redux_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -52,6 +52,8 @@ static void redux_cuda_kernel(void *descr[], void *arg)
 
 static void neutral_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -65,6 +67,8 @@ static void neutral_cuda_kernel(void *descr[], void *arg)
 #ifdef STARPU_USE_OPENCL
 static void redux_opencl_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned h_dst, h_src;
@@ -86,6 +90,8 @@ static void redux_opencl_kernel(void *descr[], void *arg)
 
 static void neutral_opencl_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned h_dst = 0;
@@ -100,6 +106,8 @@ static void neutral_opencl_kernel(void *descr[], void *arg)
 
 void redux_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -109,6 +117,8 @@ void redux_cpu_kernel(void *descr[], void *arg)
 
 void neutral_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -153,8 +163,9 @@ static struct starpu_codelet neutral_cl =
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+static void increment_opencl_kernel(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem d_token = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -173,6 +184,7 @@ static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE
 #ifdef STARPU_USE_CUDA
 static void increment_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -190,6 +202,7 @@ static void increment_cuda_kernel(void *descr[], void *arg)
 
 void increment_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
