@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +13,8 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
-#include <config.h>
+
 #include <starpu.h>
-#include <starpu_opencl.h>
 #include "../test_interfaces.h"
 
 #define KERNEL_LOCATION "tests/datawizard/interfaces/bcsr/bcsr_opencl_kernel.cl"
@@ -39,15 +38,17 @@ test_bcsr_opencl_func(void *buffers[], void *args)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_load_opencl_from_file");
 
 	uint32_t nnz = STARPU_BCSR_GET_NNZ(buffers[0]);
+ 	uint32_t r   = ((struct starpu_bcsr_interface *)buffers[0])->r;
+ 	uint32_t c   = ((struct starpu_bcsr_interface *)buffers[0])->c;
+	nnz *= (r*c);
 	cl_mem nzval = (cl_mem)STARPU_BCSR_GET_NZVAL(buffers[0]);
 
 	cl_context context;
-	id = starpu_worker_get_id();
+	id = starpu_worker_get_id_check();
 	devid = starpu_worker_get_devid(id);
 	starpu_opencl_get_context(devid, &context);
 
-	cl_mem fail = clCreateBuffer(context, CL_MEM_COPY_HOST_PTR,
-		sizeof(int), &bcsr_config.copy_failed, &err);
+	cl_mem fail = clCreateBuffer(context, CL_MEM_COPY_HOST_PTR, sizeof(int), &bcsr_config.copy_failed, &err);
 
 	if (err != CL_SUCCESS)
 		STARPU_OPENCL_REPORT_ERROR(err);
