@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2012-2013,2015-2017                      CNRS
- * Copyright (C) 2011,2013-2014,2016                      Université de Bordeaux
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,6 +24,7 @@
 
 void scal_func_cpu(void *buffers[], void *cl_arg)
 {
+	(void)cl_arg;
 	unsigned i;
 
 	struct starpu_vector_interface *vector = (struct starpu_vector_interface *) buffers[0];
@@ -40,8 +39,9 @@ void scal_func_cpu(void *buffers[], void *cl_arg)
 #ifdef STARPU_USE_OPENCL
 struct starpu_opencl_program opencl_program;
 
-void scal_func_opencl(void *buffers[], void *_args)
+void scal_func_opencl(void *buffers[], void *cl_arg)
 {
+	(void)cl_arg;
 	int id, devid;
         cl_int err;
 	cl_kernel kernel;
