@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2013,2015                           CNRS
- * Copyright (C) 2009-2010,2012-2016                      Université de Bordeaux
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -50,6 +48,8 @@ static unsigned *v;
 
 static void callback(void *arg)
 {
+	(void)arg;
+
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
 	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
@@ -63,8 +63,10 @@ static void callback(void *arg)
 	}
 }
 
-void codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 //	int id = starpu_worker_get_id();
 //	FPRINTF(stderr, "worker #%d\n", id);
 }
