@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012, 2014-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +15,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include <limits.h>
 #include <unistd.h>
@@ -32,8 +31,11 @@
 #endif
 #define VECTORSIZE	1024
 
-void codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	int worker_size = starpu_combined_worker_get_size();
@@ -59,7 +61,7 @@ static struct starpu_codelet cl =
 };
 
 
-int main(int argc, char **argv)
+int main(void)
 {
 	starpu_data_handle_t v_handle;
 	unsigned *v;
@@ -95,7 +97,7 @@ int main(int argc, char **argv)
 			task->workerid = worker;
 
 			ret = starpu_task_submit(task);
-			if (ret == -ENODEV) goto enodev;
+			if (ret == -ENODEV) { task->destroy = 0; starpu_task_destroy(task); goto enodev; }
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 		}
 	}
@@ -112,7 +114,7 @@ int main(int argc, char **argv)
 enodev:
 	starpu_data_unregister(v_handle);
 	starpu_free(v);
-	fprintf(stderr, "WARNING: No one can execute this task\n");
+	fprintf(stderr, "WARNING: No one can execute the task on workerid %d\n", worker);
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
 	starpu_shutdown();
